package es.unican.docencia.y2011.eda.practica2.estatico;

import java.io.BufferedReader;
import java.io.Console;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * Tipo enumerado para representar los dias de la semana. La clase
 * no asume la existencia de ninguna clase padre que proporcione la
 * implementacion predefinida de ningn metodo
 * 
 * @author Pablo Sanchez (p.sanchez@unican.es)
 * 		   http://personales.unican.es/sanchezbp	
 *
 * ADVERTENCIA: Se han suprimido todos los acentos en los comentarios
 *   para evitar problemas de compatibilidad entre diferentes 
 *   computadoras 
 */
public enum DiaSemana {
	
	//--------------------------------------------------------//
	// Declaracion de los valores constantes para el tipo     //
	//--------------------------------------------------------//	
	Lunes, Martes, Miercoles, Jueves, Viernes, Sabado, Domingo;
	
	protected final static int MAX_DIAS_SEMANA = 7; 
	
	//---------------------------------------------------------//
	// Estos metodos son absurdos, pero se incluyen para que   //
	// se entienda cual es la funcion de un metodo generador   //
	// Tendria sentido si quisiesemos hacer los valores de los //
	// enumerados privados, pero no hay ninguna razon para	   //
	// ello, normalmente	                                   //
	//---------------------------------------------------------//
	
	/**
	 * Devuelve en valor enumerado correspondiente al Lunes
	 * @return Lunes
	 */
	public static DiaSemana Lunes() {
		return Lunes;
	} // Lunes 

	/**
	 * Devuelve en valor enumerado correspondiente al Martes
	 * @return Martes	 
	 */
	public static DiaSemana Martes() {
		return Martes;
	} // Martes

	/**
	 * Devuelve en valor enumerado correspondiente al Miercoles
	 * @return Miercoles
	 */
	public static DiaSemana Miercoles() {
		return Miercoles;
	} // Miercoles

	/**
	 * Devuelve en valor enumerado correspondiente al Jueves
	 * @return Jueves
	 */
	public static DiaSemana Jueves() {
		return Jueves;
	} // Jueves

	/**
	 * Devuelve en valor enumerado correspondiente al Viernes
	 * @return Viernes
	 */
	public static DiaSemana Viernes() {
		return Viernes;
	} // Viernes

	/**
	 * Devuelve en valor enumerado correspondiente al Sabado
	 * @return Sabado
	 */
	public static DiaSemana Sabado() {
		return Sabado;
	} // Sabado

	/**
	 * Devuelve en valor enumerado correspondiente al Domingo
	 * @return Domingo
	 */
	public static DiaSemana Domingo() {
		return Domingo;
	}  // Domingo
	
	//-----------------------------------------------------------//
	// Funciones auxiliares para manipular el tipo               //
	// (Helper functions)
	//-----------------------------------------------------------//
	/**
	 * Devuelve el entero correspondiente al entero asociado a la 
	 * sucesion de dias de la semana de acuerdo al orden natural de 
	 * los dias de la semana comenzando en Lunes 
	 * @param d Dia de la semana cuyo numero de orden queremos conocer 
	 * 			Pre: d != null
	 * @result Posicion de dicho dia en la sucesion de dias de la semana 
	 *         comenzando en Lunes
	 */
	protected static int diaSemana2Int(DiaSemana d) {
		
		int result = 0; 
		
		switch(d) {
			case Lunes: 
				result = 0;break;
			case Martes: 
				result = 1;break;
			case Miercoles: 
				result = 2;break;
			case Jueves: 
				result = 3;break;
			case Viernes: 
				result = 4;break;
			case Sabado: 
				result = 5;break;
			case Domingo: 
				result = 6;break;
			default:
				throw new IllegalArgumentException("Valor enumerado no soportado");
		} //switch
		
		return result;
	} // diaSemana2Int
	
	/**
	 * Devuelve el de la semana correspondiente al entero asociado a la 
	 * sucesion de dias de la semana de acuerdo al orden natural de 
	 * los dias de la semana comenzando en Lunes 
	 * @param i Entero para cuya posicin deseamos obtener el da de la semana
	 * 			Pre: ((0 <= i) && (i < MAX_DIAS_SEMANA))
	 * @result Dia de la semana correspondiente al dia i-esimo en la sucesion de 
	 * 		   dias de la semana comenzando en Lunes
	 */
	protected static DiaSemana int2diaSemana(int i) {
		
		DiaSemana result = Lunes;
		
		switch(i) {
			case 0: 
				result = Lunes;break;
			case 1: 
				result = Martes;break;
			case 2: 
				result = Miercoles;break;
			case 3: 
				result = Jueves;break;
			case 4: 
				result = Viernes;break;
			case 5: 
				result = Sabado;break;
			case 6: 
				result = Domingo;break;
			default:
				throw new IllegalArgumentException("No existe valor enumerado correspondiente al entero " + i);
		} //switch
		
		return result;
		
	} // int2diaSemana
	
	//-----------------------------------------------------------//
	// Funciones para manipular el tipo                          //
	//-----------------------------------------------------------// 
	
	/**
	 *  Devuelve el dia siguiente al dia que se pasa como parametro
	 * @param d Dia de la semana sobre el que se calcula el siguiente
	 * 			Pre: d != null
	 * @return Dia siguiente al dia d 
	 */
	public static DiaSemana siguiente(DiaSemana d) {
		return int2diaSemana((diaSemana2Int(d) + 1) % DiaSemana.MAX_DIAS_SEMANA);
	} // siguiente
	
	/**
	 *  Devuelve el dia anterior al dia que se pasa como parametro
	 * @param d Dia de la semana sobre el que se calcula el anterior
	 * 			Pre: d != null
	 * @return Dia anterior al dia d 
	 */
	public static DiaSemana anterior(DiaSemana d) {
		return int2diaSemana((diaSemana2Int(d) - 1 + 7) % DiaSemana.MAX_DIAS_SEMANA);
	} // anterior
	
	/**
	 *  Devuelve verdadero si el da es un da habil. Habiles son todos 
	 *  menos los sbados y domingos (depende de cada caso)  
	 * @param d Dia de la semana que se quieres saber si es hbil o no
	 * 			Pre: d != null 
	 * @return Verdadero si d es un da hbil, falso en cualquier otro caso  
	 */
	public static boolean esHabil(DiaSemana d) {
		return (diaSemana2Int(d) <= diaSemana2Int(Viernes));
	} // esLaboral

	/**
	 * Dia de la semana correspondiente a num dias despues del da d que se 
	 * pasa como parmetro  
	 * @param d Dia a partir del cual se inicia el calculo
	 * 			Pre: d != null 
	 * @param num Numero de dias contar desde el dia d. 
	 * 			Pre: i >= 0 
	 * @return Dia de la semana correspondiente a num dias despues del da d 
	 * @throws IllegalArgumentException Si el valor de das a sumar fuese negativo
	 */
	public static DiaSemana suma(DiaSemana d, int num) throws IllegalArgumentException { 
		if (num < 0) {
			throw new IllegalArgumentException();
		} // if
		return int2diaSemana((diaSemana2Int(d) + num) % DiaSemana.MAX_DIAS_SEMANA);
	} //  suma
	
	/**
	 * Devuelve verdadero si el primer parmetro est antes del segundo en la lista
	 * de das de la semana. La semana comienza en Lunes.  
	 * @param d1 Primer argumento
	 * 			 Pre: d1 != null  
	 * @param d2 Segundo argumento
	 * 			 Pre: d2 != null
	 * @return Verdadero si d1 < d2 de acuerdo a la lista de los dias de la semana comenzando
	 * 		   en lunes	  
	 */
	public static boolean menor(DiaSemana d1, DiaSemana d2) {
		return diaSemana2Int(d1) < diaSemana2Int(d2); 
	} // menor
	
	/**
	 * Devuelve verdadero si el primer parmetro est despus del segundo en la lista
	 * de das de la semana. La semana comienza en Lunes.  
	 * @param d1 Primer argumento
	 * 			 Pre: d1 != null  
	 * @param d2 Segundo argumento
	 * 			 Pre: d2 != null
	 * @return Verdadero si d1 > d2 de acuerdo a la lista de los dias de la semana comenzando
	 * 		   en lunes	  
	 */
	public static boolean mayor(DiaSemana d1, DiaSemana d2) {
		return !(menor(d1,d2) || (d1 == d2)); 
	} // menor
	
	/**
	 * Devuelve verdadero si el primer parmetro est antes del segundo en la lista
	 * de das de la semana, o si ambos elementos son iguales. La semana comienza en Lunes.  
	 * @param d1 Primer argumento  
	 * 			 Pre: d1 != null
	 * @param d2 Segundo argumento
	 * 			 Pre: d2 != null
	 * @return Verdadero si d1 <= d2 de acuerdo a la lista de los dias de la semana comenzando
	 * 		   en lunes	  
	 */
	public static boolean menorIgual(DiaSemana d1, DiaSemana d2) {
		return (menor(d1,d2) || (d1 == d2)); 
	} // menor

	/**
	 * Devuelve verdadero si el primer parmetro est despus del segundo en la lista
	 * de das de la semana, o si ambos elementos son iguales. La semana comienza en Lunes.  
	 * @param d1 Primer argumento  
	 * 			 Pre: d1 != null
	 * @param d2 Segundo argumento
	 * 			 Pre: d2 != null
	 * @return Verdadero si d1 >= d2 de acuerdo a la lista de los dias de la semana comenzando
	 * 		   en lunes	  
	 */
	public static boolean mayorIgual(DiaSemana d1, DiaSemana d2) {
		return (mayor(d1,d2) || (d1 == d2)); 
	} // menor
	
	//-----------------------------------------------------------//
	// Funciones de entrada salida								 //
	//-----------------------------------------------------------//
	
	/**
	 * Devuelve una cadena con el nombre asociado a cada valor enumerado
	 * @param d Dia de la semana del cual queremos conocer su nombre.
	 * 			Pre: d != null 
	 * @return Cadena conteniendo el nombre del da de la semana d
	 */
	public static String toString(DiaSemana d) {
		
		String result = null; 

		switch(d) {
		case Lunes: 
			result = "Lunes";break;
		case Martes: 
			result = "Martes";break;
		case Miercoles: 
			result = "Miercoles";break;
		case Jueves: 
			result = "Jueves";break;
		case Viernes: 
			result = "Viernes";break;
		case Sabado: 
			result = "Sabado";break;
		case Domingo: 
			result = "Domingo";break;
		default:
			throw new IllegalArgumentException("Valor enumerado no soportado");
		} //switch		
		
		return result;		
		
	} // toString
	
	/**
	 * Procesa una cadena de caracteres para convertirla a un valor DiaSemana
	 * @param s Cadena de caracteres conteniendo un dia de la semana
	 * @return Dia de la semana correspondinte a la cadena pasada como argumento 
	 * @throws IllegalArgumentException Se lanza esta excepcin cuando la cadena no contiene un 
	 * 								    valor vlido
	 */
	protected static DiaSemana parseDiaSemana(String s) throws IllegalArgumentException {
		
		DiaSemana d = Lunes;
		
		if (s.equals("Lunes")) {
			d = Lunes;
		} else if (s.equals("Martes")) {
			d  = Martes;
		} else if (s.equals("Miercoles")) {
			d = Miercoles;
		} else if (s.equals("Jueves")) {
			d = Jueves;
		} else if (s.equals("Viernes")) {
			d = Viernes;
		} else if (s.equals("Sabado")) {
			d = Sabado;
		} else if (s.equals("Domingo")) {
			d = Domingo; 
		} else {
			throw new IllegalArgumentException();
		} // if	
		
		return d;
		
	} // parseDiaSemana
	
	/**
	 * Lee por la consola el valor de un dia de la semana introducido como una cadena de caracteres
	 * Los valores vlidos son Lunes, Martes, Miercoles, Jueves, Viernes, Sabado y Domingo. 
	 * @param mensaje Mensaje que es mostrado por pantalla antes de pedir el valor del dia de la semana  
	 * @return Dia de la semana leido por teclado
	 * @throws IOException Si se produce un error de entrada/salida al leer 
	 */
	public static DiaSemana LeerDiaSemana(String mensaje) throws IOException {
		
		DiaSemana d = Lunes;
		
		// Creamos una consola para poder leer valores de la consola
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		
		boolean ok;
		
		do {
			ok = true;
			System.out.print(mensaje);
			String s = br.readLine();
			try {
				d = parseDiaSemana(s);
			} catch (IllegalArgumentException e) {
				ok = false;
				System.out.println("Valor introducido incorrecto, introduzcalo de nuevo.");
			} // try-catch
		} while (!ok);	
		
		return d;
		
	} // LeerDiaSemana


} // enum DiaSemana